#!/usr/bin/python
# vim: fileencoding=utf-8
"""bigstring.py

"""
#   kuechenstation - a curses client for xmms2
#   
#   Copyright (C) 2008  Jan Eike von Seggern <eikevons@yahoo.de>
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import subprocess
import logging

class FigletFrontend(object):
    figpath = "figlet"
    figfont = "standard"
    encoding = "UTF-8"
    def __init__(self, udata):
        if type(udata) != unicode:
            udata = unicode(udata)
        self.__data = []
        self.__length = 0
        self.__height = 0
        args = [self.figpath,
                '-C', 'utf8',
                '-f', self.figfont,
                '-w', '1000',
                udata.encode(self.encoding)]
        logging.debug("running figlet: %s"%(args))
        figlet = subprocess.Popen(args, stdout=subprocess.PIPE)
        if figlet.wait() != 0:
            raise RuntimeError('Failed to run figlet!')

        for l in figlet.stdout:
            self.__data.append(unicode(l[:-1], self.encoding))
            self.__length = max(self.__length, len(l) - 1)
        self.__height = len(self.__data)

    def __str__(self):
        s = ''
        for ul in self.__data:
            s += ul.encode(self.encoding) + '\n'
        return s

    def __len__(self):
        return self.__length

    length = property(__len__)
    height = property(lambda self: self.__height)
    data = property(lambda self: self.__data)


class WindowedMLS(FigletFrontend):
    def __init__(self, udata, index):
        super(WindowedMLS, self).__init__(udata)
        self.__index = index

    def getWindow(self, width):
        """Return a list with strings."""
        r = []
        for line in self.data:
            new = line[self.__index:]
            while len(new) < width:
                new += line
            r.append(new[:width])
        return r

    def step(self):
        self.__index += 1
        if self.__index >= self.length:
            self.__index = 0

    index = property(lambda self: self.__index)


# this is for testing
if __name__ == '__main__':
    import time

    sms = WindowedMLS(u"Täster * ")
    print "length: %i"%(len(sms))
    while True:
        print "**", sms.index, "**"
        for line in sms.getWindow(100):
            print "  ", len(line), line
        sms.step()
        time.sleep(.2)
