# vim:fileencoding=utf-8
"""eventframework.py

event handling methods must take event as argument:
>>> def exampleHandler(self, event):
>>>    pass
"""
#   kuechenstation - a curses client for xmms2
#   
#   Copyright (C) 2008  Jan Eike von Seggern <eikevons@yahoo.de>
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import threading
import logging


class EventError(Exception):
    pass


class EventRecipient(object):
    __active = False
    def __init__(self, parent, distributor):
        self.__parent = parent
        self.__dist = distributor
        self.__events = {}
        # logging.debug("created EventRecipient instance of class %s:\n"
                      # " parent = %s\n"
                      # " distri = %s\n"%(self.__class__, parent, distributor))

    def registerEvent(self, method, *events):
        for ev in events:
            if type(ev) is str and len(ev) == 1:
                ev = ord(ev)
            elif type(ev) is not int:
                EventError("Cannot register event '%s': Must be int"
                           " or length 1 str!"%(ev))
            if method is None:
                del self.__events[ev]
            else:
                self.__events[ev] = method

    def handleEvent(self, event):
        # logging.error("EventRecipient '%s' handling event %s"%(
            # self, event))
        if self.__events.has_key(event):
            self.__events[event](event)
        elif self.__parent is not None:
            self.__parent.handleEvent(event)

    def activate(self):
        if self.__active:
            return
        # logging.debug("activating %s"%(self))
        self.__active = True
        self.__dist.occupy(self)

    def deactivate(self, parent_activate = True):
        if not self.__active:
            return
        # logging.debug("deactivating %s"%(self))
        self.__active = False
        if parent_activate and self.parent:
            self.parent.activate()

    parent = property(lambda self: self.__parent)
    distributor = property(lambda self: self.__dist)
    active = property(lambda self: self.__active)


class EventDistributor(object):
    def __init__(self, init_recip):
        self.__recipient = init_recip
        self.__working = threading.Lock()

    recipient = property(lambda self: self.__recipient)

    def occupy(self, new_recip):
        logging.debug("EventDistributor occupied by %s"%(new_recip))
        self.__recipient.deactivate(parent_activate=False)
        self.__recipient = new_recip

    def distributeEvent(self, event):
        self.__working.acquire()
        logging.debug("EventRecipient '%s' gets event '%s'"%(
                      self.__recipient.__class__.__name__, event))
        self.__recipient.handleEvent(event)
        self.__working.release()
