# vim:fileencoding=utf-8
"""helpers.py

little helper functions used in different parts of the program
"""

import urllib2
import curses

def ms2string(ms):
    "Return a formated time string."
    if ms is None:
        return "unknown"

    hours = ms/(1000*60*60)
    ms %=(1000*60*60)

    mins = ms/(1000*60)
    ms %=(1000*60)

    secs = ms/1000
    if hours:
        return "%i:%02i:%02i"%(hours, mins, secs)
    else:
        return "%i:%02i"%(mins, secs)

def convColor(cval):
    """Return a pair of curses color constants.

    bla-> black
    b  -> blue
    w  -> white
    r  -> red
    g  -> green
    y  -> yellow
    c  -> cyan
    m  -> magenta
    t  -> transparent
    """
    fg, bg = cval.lower().split()
    def getconstant(col):
        if col.startswith("bla"):
            return curses.COLOR_BLACK
        if col.startswith("w"):
            return curses.COLOR_WHITE
        if col.startswith("b"):
            return curses.COLOR_BLUE
        if col.startswith("r"):
            return curses.COLOR_RED
        if col.startswith("g"):
            return curses.COLOR_GREEN
        if col.startswith("y"):
            return curses.COLOR_YELLOW
        if col.startswith("c"):
            return curses.COLOR_CYAN
        if col.startswith("m"):
            return curses.COLOR_MAGENTA
        if col.startswith("t"):
            return -1
        return curses.COLOR_WHITE
    return (getconstant(fg), getconstant(bg))

def treatURL(url):
    """Replace '+' -> ' ' and unquot %-quotes."""
    url = url.replace(u"+", u" ")
    url = urllib2.unquote(url)
    return url


# curses color pair numbers
# 1 -> default
# 2 -> scrolling title
# 3 -> status line
# 4 -> playlist
# 5 -> media lib

def convAttr(cval):
    """
    r -> reverse
    b -> bold
    u -> underline
    bl-> blink
    """
    if cval.startswith("r"):
        return curses.A_REVERSE
    if cval.startswith("bl"):
        return curses.A_BLINK
    if cval.startswith("b"):
        return curses.A_BOLD
    if cval.startswith("u"):
        return curses.A_UNDERLINE
    return 0

def needsitems(fun):
    """Decorator for ListWindow event methods which need a filled list."""
    def new(self, event):
        if len(self.items) == 0:
            self.message("List empty")
            return
        return fun(self, event)
    return new
