# vim:fileencoding=utf-8
"""loggingwindow.py

"""
#   kuechenstation - a curses client for xmms2
#   
#   Copyright (C) 2008  Jan Eike von Seggern <eikevons@yahoo.de>
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

from logging.handlers import BufferingHandler
from basewindows import MultiWindow
import sys, traceback

class MyHandler(BufferingHandler):
    __targetwindow = None
    def setTargetWindow(self, win):
        self.__targetwindow = win

    def shouldFlush(self, rec):
        if self.__targetwindow is None:
            return BufferingHandler.shouldFlush(self, rec)
        return True

    def flush(self):
        if self.__targetwindow is not None:
            t = [self.format(rec) for rec in self.buffer]
            self.__targetwindow.addMessages(*t)

            # for rec in self.buffer:
                # self.__targetwindow.addMessages(self.format(rec))
            # self.__targetwindow.addMessages(map(self.format, self.buffer))
        BufferingHandler.flush(self)


class LoggingWindow(MultiWindow):
    __items = ["just created"]
    # def __init__(self, *mwargs):
        # super(LoggingWindow, self).__init__(*mwargs)

        # old = sys.excepthook
        # def new_ehook(ec, ei, tb):
            # self.exceptionHook(ec, ei, tb)
            # old(ec, ei, tb)
        # sys.excepthook = new_ehook

    def exceptionHook(self, ex_cl, ex_i, tb):
        self.addMessages('Uncaught exception:')
        self.addMessages(traceback.format_exception(ex_cl, ex_i, tb))

    def addMessages(self, *messages):
        for m in messages:
            while m:
                self.__items.append(m[:self.width])
                m = m[self.width:]

        # self.__items += messages
        while len(self.__items) > self.height:
            self.__items.pop(0)
        self.refresh()

    def refresh(self):
        if not self.active:
            return
        if self.mainapp.quitflag.isSet():
            return

        self.window.clear()
        limit = min(self.height, len(self.__items))
        for index in xrange(limit):
            l = self.__items[index]
            if len(l) > self.width:
                l = l[:self.width]
            else:
                l = l.ljust(self.width)
            try:
                # don't use addLine, addStr here, because they log themselve
                self.window.addstr(index, 0, l, self.colorattr)
            except:
                break
        super(LoggingWindow, self).refresh()
