# vim:fileencoding=utf-8
"""playlistwindow.py

"""
#   kuechenstation - a curses client for xmms2
#   
#   Copyright (C) 2008  Jan Eike von Seggern <eikevons@yahoo.de>
#   
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#   
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#   
#   You should have received a copy of the GNU General Public License along
#   with this program; if not, write to the Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import curses
import logging
import ConfigParser
import os
import time
import xmmsclient

from basewindows import ListWindow, SizeError, ListItem
from kuechenwindows import TrackInfoWindow
from helpers import ms2string, needsitems


class PlaylistItem(ListItem):
    def getString(self, width):
        if self.selected:
            mark = u"*"
        else:
            mark = u" "
        if self.data.has_key('title'):
            l = u"%s %s - %02i - %s"%(mark, self.data.get('artist', 'Unkown'),
                               self.data.get('tracknr', 0),
                               self.data.get('title', 'Unknown'))
        else:
            url = self.data.get('url')
            if url.startswith(u"file"):
                url = os.path.basename(url)
            l = u"%s %s"%(mark, url)

        l = l.encode(self.encoding, 'replace')

        # do all the size stuff with encoded ascii strings
        if len(l) < width:
            l = l.ljust(width)
        elif len(l) > width:
            l = l[:width]
        timestr = ms2string(self.data.get('duration'))
        l = l[:-len(timestr)] + timestr
        return l

    def __str__(self):
        return "<PlaylistItem '%s'>"%(self.getString(20))


class PlaylistWindow(ListWindow):
    help = ("space: select  J/K: move up/down  x: remove"
                "  enter: jump&play  c: clear")
    option_key = "playlist"
    color_pair = 6
    # bool indicating, we don't want to refresh for a moment
    # e.g. while performing some xmms2 actions
    displaylock = False
    def __init__(self, *multiwinargs):
        super(PlaylistWindow, self).__init__(*multiwinargs)

        # the list index, which is currently played
        self.__curplay = None

        self.__init_register_events()

        self.playattr = self.mainapp.getAttrOption(self.name, "playing attr")
        # if self.playattr == 0:
            # self.playattr = curses.A_BOLD

    def __init_register_events(self):
        rE = self.registerEvent

        rE(self.evReload, 'r')
        rE(self.evRemove, 'x', 'd')
        # 11 => ^K
        rE(self.evRemoveToEnd, 'D', 11)
        # 21 => ^U
        rE(self.evRemoveToTop, 'X', 21)
        rE(self.evMoveUp, 'K')
        rE(self.evMoveDown, 'J')
        rE(self.evClear, 'c')

        rE(self.evPasteBefore, "P")
        rE(self.evPasteAfter, "p")
        rE(self.evYank, "y")

        rE(self.evPlay, '\n')
        rE(self.evInfo, 'i')

    def activate(self):
        """reloads playlist from xmms"""
        super(PlaylistWindow, self).activate()
        logging.debug("activated %s: self.active = %s"%(self, self.active))
        r = self.xmms.playlist_list_entries()
        r.wait()
        l = r.value()
        self.items = []
        for mid in l:
            r2 = self.xmms.medialib_get_info(mid)
            r2.wait()
            pd = r2.value()
            self.items.append(PlaylistItem(pd, self.encoding))

        r = self.xmms.playlist_current_pos()
        r.wait()
        v = r.value()
        if v:
            self.__curplay = v["position"]

    def refresh(self):
        """refreshes display"""
        if not self.active or self.displaylock:
            return
        if self.__curplay is None:
            r = self.xmms.playlist_current_pos()
            r.wait()
            v = r.value()
            if v:
                self.__curplay = v["position"]

        super(PlaylistWindow, self).refresh()

    def addAttr(self, index, attr):
        if index == self.__curplay:
            attr |= self.playattr
        return super(PlaylistWindow, self).addAttr(index, attr)

    def displayItem(self, index, y=None, attr=0):
        if y is None:
            y = index - self.offset
        super(PlaylistWindow, self).displayItem(index, y, attr)

    def selectedIndices(self):
        indices = [i for i, item in enumerate(self.items) if item.selected]
        if len(indices) == 0:
            return [self.cursel]
        else:
            return indices

    def removeItems(self, rm_indices):
        # fill the copybuffer
        infos = []
        for i in rm_indices:
            infos.append(self.items[i].data)
        self.mainapp.copybuffer = infos

        # count removed items so we get indices right
        rm_cnt = 0
        self.message("removing item(s) %s"%(rm_indices))
        self.displaylock = True
        for i in rm_indices:
            self.xmms.playlist_remove_entry(i - rm_cnt).wait()
            rm_cnt += 1
        self.displaylock = False
        self.message("removed item(s) %s"%(rm_indices))
        self.refresh()


    # xmms callbacks
    def cbPlaylistCurrentPos(self, result):
        if not self.active:
            return
        time.sleep(.5)
        d = result.value()
        if d["name"] == u"Default":
            self.__curplay = d["position"]
            self.refresh()

    def cbPlaylistChanged(self, result):
        logging.debug("Called cbPlaylistChanged(%s)"%(result.value()))
        if not self.active:
            return

        pd = result.value()
        if pd.get('type') == xmmsclient.PLAYLIST_CHANGED_REMOVE:
            index = pd.get('position')
            logging.debug("item #%i: %s"%(index,
                self.items[index].getString(20)))
            logging.debug("removing item %i"%(index))

            old = self.items.pop(pd.get('position'))
            logging.debug("item #%i: %s"%(index, old.getString(30)))

        elif pd.get('type') == xmmsclient.PLAYLIST_CHANGED_ADD:
            res2 = self.xmms.medialib_get_info(pd.get('id'))
            res2.wait()
            self.items.insert(pd.get('position'),
                              PlaylistItem(res2.value(), self.encoding))

        elif pd.get('type') == xmmsclient.PLAYLIST_CHANGED_MOVE:
            oldpos = pd.get('position')
            newpos =  pd.get('newposition')
            logging.debug("item #%i: %s"%(oldpos,
                self.items[oldpos].getString(20)))
            logging.debug("item #%i: %s"%(newpos,
                self.items[newpos].getString(20)))
            logging.debug("moving item %i to %i"%(oldpos, newpos))

            pi = self.items.pop(oldpos)
            self.items.insert(newpos, pi)

            logging.debug("item #%i: %s"%(oldpos,
                self.items[oldpos].getString(20)))
            logging.debug("item #%i: %s"%(newpos,
                self.items[newpos].getString(20)))
        else:
            # in all other cases rebuild the playlist from ground up
            self.activate()

        self.refresh()


    # events
    @needsitems
    def evPlay(self, event):
        self.xmms.playlist_set_next(self.cursel).wait()
        self.xmms.playback_tickle().wait()
        r = self.xmms.playback_status()
        r.wait()
        if r.value() != xmmsclient.PLAYBACK_STATUS_PLAY:
            self.xmms.playback_start().wait()

    def evReload(self, event):
        self.activate()
        self.refresh()
        self.message("reloaded playlist")

    @needsitems
    def evClear(self, event):
        self.xmms.playlist_clear().wait()
        self.evReload(None)

    @needsitems
    def evRemove(self, event):
        rm_indices = self.selectedIndices()
        self.removeItems(rm_indices)

    @needsitems
    def evRemoveToEnd(self, event):
        rm_indices = range(self.cursel, len(self.items))
        self.removeItems(rm_indices)

    @needsitems
    def evRemoveToTop(self, event):
        rm_indices = range(0, self.cursel + 1)
        self.cursel = 0
        self.removeItems(rm_indices)

    @needsitems
    def evMoveUp(self, event):
        sel = self.selectedIndices()
        # don't move over the top, this works because sel is already sorted
        if sel[0] == 0:
            return
        self.cursel -= 1
        self.repairIndices()

        self.message("moving item(s) %s"%(sel))
        self.displaylock = True
        for i in sel:
            self.xmms.playlist_move(i, i-1)
        self.displaylock = False
        self.message("moved item(s) %s"%(sel))
        self.refresh()

    @needsitems
    def evMoveDown(self, event):
        sel = self.selectedIndices()
        # don't move out of the list, this works because sel is already
        # sorted
        if sel[-1] == len(self.items) - 1:
            return
        self.cursel += 1
        self.repairIndices()

        self.message("moving item(s) %s"%(sel))
        self.displaylock = True
        for i in reversed(sel):
            self.xmms.playlist_move(i, i+1)
        self.displaylock = False
        self.message("moved item(s) %s"%(sel))
        self.refresh()

    @needsitems
    def evPosHome(self, event):
        if self.__curplay is None:
            r = self.xmms.playlist_current_pos()
            r.wait()
            v = r.value()
            if v:
                self.__curplay = v["position"]
            else:
                self.__curplay = 0
        self.cursel = self.__curplay
        if self.cursel >= self.height + self.offset:
            self.offset = self.cursel + self.height - 1
        logging.info("before refresh: offset %i  cursel %i"%(
            self.offset, self.cursel))
        self.refresh()
        logging.info("after refresh: offset %i  cursel %i"%(
            self.offset, self.cursel))
        self.message("cursor home")

    @needsitems
    def evInfo(self, event):
        tiw = TrackInfoWindow("trackinfo", self, self.distributor,
                self.window, self.mainapp, self.items[self.cursel].data)
        tiw.activate()
        tiw.refresh()

    def evPasteAfter(self, event):
        if len(self.mainapp.copybuffer) == 0:
            self.message("no tracks to paste!")
            return
        ids = [info["id"] for info in self.mainapp.copybuffer]

        self.displaylock = True
        if len(self.items) != 0:
            for offset, mlibid in enumerate(ids):
                self.xmms.playlist_insert_id(self.cursel + offset + 1, mlibid)
        else:
            for mlibid in ids:
                self.xmms.playlist_add_id(mlibid)
        self.displaylock = False

        self.message("inserted id(s) %s"%(ids))
        self.refresh()

    def evPasteBefore(self, event):
        if len(self.mainapp.copybuffer) == 0:
            self.message("no tracks to paste!")
            return
        ids = [info["id"] for info in self.mainapp.copybuffer]

        self.displaylock = True
        if len(self.items) != 0:
            for offset, mlibid in enumerate(ids):
                self.xmms.playlist_insert_id(self.cursel + offset, mlibid)
        else:
            for mlibid in ids:
                self.xmms.playlist_add_id(mlibid)
        self.cursel += len(ids)
        self.displaylock = False

        self.message("inserted id(s) %s"%(ids))
        self.refresh()

    @needsitems
    def evYank(self, event):
        selected = list()
        for idx in self.selectedIndices():
            selected.append(self.items[idx].data)
        self.mainapp.copybuffer = selected
        self.message("yanked %s" % self.selectedIndices())
